﻿using Newtonsoft.Json;
using RetSoft.Demo.Desktop.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Windows.Forms;

namespace RetSoft.Demo.Desktop
{
    public partial class DocumentDemo : Form
    {
        private readonly HttpClient client;

        public DocumentDemo(HttpClient client)
        {
            InitializeComponent();
            this.client = client;
        }

        #region Helper Methods
        private string ConstructTempFileName(string name, string fileExt)
        {
            // Validate filename by escaping invalid characters
            const char escapeChar = '_';
            var fileName = name.Trim();
            Path.GetInvalidFileNameChars().ToList().ForEach(x => fileName = fileName.Replace(x, escapeChar));
            // Append file extension if not already part of name
            if (!fileName.ToLower().EndsWith(fileExt.ToLower()))
            {
                fileName += fileExt;
            }
            // Place in temp directory
            return Path.Combine(Path.GetTempPath(), fileName);
        }
        #endregion

        private async void DownloadDocument(bool favorPDF)
        {
            if (listViewDocuments.SelectedItems.Count == 0)
                return;

            try
            {
                Enabled = false;

                var documentId = int.Parse(listViewDocuments.SelectedItems[0].Text);
                // Read document information
                var response = await client.GetAsync($"api/documents/{documentId}");
                response.EnsureSuccessStatusCode();
                var document = await response.Content.ReadAsJsonAsync<DocumentResponse>();
                // Read document data (FormFile)
                response = await client.GetAsync($"api/documents/{documentId}" + (favorPDF ? "/data/favorpdf" : "/data"));
                response.EnsureSuccessStatusCode();
                var data = await response.Content.ReadAsByteArrayAsync();

                // Construct temporary filename
                var fileName = response.Content.Headers.ContentDisposition?.FileName?.Trim('\"');
                var fileExt = string.Empty;
                if (string.IsNullOrEmpty(fileName))
                {
                    fileName = document.Name;
                }
                else
                {
                    fileExt = Path.GetExtension(fileName);
                }
                if (string.IsNullOrEmpty(fileExt))
                {
                    fileExt = favorPDF ? ".pdf" : document.FileTypeExtension;
                }
                // Write result
                var tempFileName = ConstructTempFileName(fileName, fileExt);
                File.WriteAllBytes(tempFileName, data);
                Process.Start(tempFileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                Enabled = true;
            }
        }

        private void btnDownloadPDF_Click(object sender, EventArgs e)
        {
            DownloadDocument(true);
        }

        private void btnDownload_Click(object sender, EventArgs e)
        {
            DownloadDocument(false);
        }

        private async void btnAddDocument_Click(object sender, EventArgs e)
        {
            try
            {
                Enabled = false;

                /// Insert new document
                string fileName = null;
                using (var f = new OpenFileDialog())
                {
                    if (f.ShowDialog() == DialogResult.OK)
                    {
                        fileName = f.FileName;
                    }
                }
                if (string.IsNullOrEmpty(fileName))
                    return;

                var parentId = 0; // Places the document at the root folder
                var name = Path.GetFileNameWithoutExtension(fileName);
                var data = File.ReadAllBytes(fileName);
                var form = new MultipartFormDataContent
                {
                    { new StringContent(name), "Name" },
                    { new ByteArrayContent(data), "File", Path.GetFileName(fileName) }
                };
                var response = await client.PostAsync($"api/folders/{parentId}/documents", form);
                response.EnsureSuccessStatusCode();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                Enabled = true;
            }

            btnGetAll.PerformClick();
        }

        private async void btnDelete_Click(object sender, EventArgs e)
        {
            if (listViewDocuments.SelectedItems.Count == 0)
                return;

            var documentId = int.Parse(listViewDocuments.SelectedItems[0].Text);
            var documentName = listViewDocuments.SelectedItems[0].SubItems[1].Text;
            if (MessageBox.Show($"Do you really want to delete '{documentName}'?", "Delete", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                var response = await client.DeleteAsync($"api/documents/{documentId}");
                response.EnsureSuccessStatusCode();
                var result = int.Parse(await response.Content.ReadAsStringAsync()); // Result represents the number of documents actually deleted (1 in this case)
                // 
                btnGetAll.PerformClick();
            }
        }

        private async void btnGetAll_Click(object sender, EventArgs e)
        {
            if (client == null)
            {
                MessageBox.Show("Not connected!");
                return;
            }

            // Read all documents from root folder
            var parentId = 0;
            var response = await client.GetAsync($"api/folders/{parentId}/documents");
            response.EnsureSuccessStatusCode();
            var documents = await response.Content.ReadAsJsonAsync<List<DocumentResponse>>();
            // Fill list
            listViewDocuments.Items.Clear();
            foreach (var item in documents)
            {
                var li = new ListViewItem(item.Id.ToString());
                li.SubItems.Add(item.Name);
                listViewDocuments.Items.Add(li);
            }
        }
    }
}
