﻿using IdentityModel.Client;
using System;
using System.Net.Http;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace RetSoft.Demo.Desktop
{
    public static class ConnectionManager
    {
        
        /// <summary>
        /// Authorize using client id / secret
        /// </summary>
        /// <param name="clientId">Client id</param>
        /// <param name="clientSecret">Client secret</param>
        /// <param name="apiUrl">Address of web api</param>
        /// <returns></returns>
        public static async Task<HttpClient> Authorize(string clientId, string clientSecret, string apiUrl)
        {
            var baseUri = new Uri(apiUrl);
            var client = new HttpClient
            {
                BaseAddress = baseUri // Base address is the web address of the api
            };

            // Gets the discovery endpoint for RetSoft APi
            var disco = client.GetDiscoveryDocumentAsync(apiUrl).Result;
            
            var tokenResponse = await client.RequestClientCredentialsTokenAsync(new ClientCredentialsTokenRequest
            {
                Address = disco.TokenEndpoint,
                ClientId = clientId,
                ClientSecret = clientSecret,
                Scope = "api.archive",
            });

            if (tokenResponse.IsError)
                throw new Exception($"Could not connect! {tokenResponse.ErrorDescription}");

            var response = await client.GetAsync("setup/ping");
            response.EnsureSuccessStatusCode();

            var status = await response.Content.ReadAsStringAsync();
            if (status != "pong")
            {
                throw new Exception($"Invalid repsonse: {status}");
            }
                
            client.SetBearerToken(tokenResponse.AccessToken);
            return client;
        }

        /// <summary>
        /// Authorize using api key
        /// </summary>
        /// <param name="apiKey">API key</param>
        /// <param name="apiUrl">Address of web api</param>
        /// <returns></returns>
        public async static Task<HttpClient> Authorize(string apiKey, string apiUrl)
        {
            var baseUri = new Uri(apiUrl);
            var client = new HttpClient
            {
                BaseAddress = baseUri
            };

            client.DefaultRequestHeaders.Add("Authorization", $"apikey {apiKey}");

            var response = await client.GetAsync("setup/ping");
            response.EnsureSuccessStatusCode();

            var status = await response.Content.ReadAsStringAsync();
            if (status != "pong")
            {
                throw new Exception($"Invalid repsonse: {status}");
            }

            return client;
        }

        /// <summary>
        /// Authorize using api key
        /// </summary>
        /// <param name="apiKey">API key</param>
        /// <param name="apiUrl">Address of web api</param>
        /// <param name="impersonate">Username</param>
        /// <returns></returns>
        public async static Task<HttpClient> Impersonate(string apiKey, string apiUrl, string impersonate)
        {
            var client = await Authorize(apiKey, apiUrl);
            if (!string.IsNullOrEmpty(impersonate))
            {
                client.DefaultRequestHeaders.Add("Impersonate", impersonate);
            }
            return client;
        }
    }
}
